@@ -30,36 +30,66 @@
 
 static char *tokudb_backup_plugin_version;
 
-static MYSQL_SYSVAR_STR(plugin_version, tokudb_backup_plugin_version, PLUGIN_VAR_NOCMDARG | PLUGIN_VAR_READONLY, "version of the tokudb backup plugin",
-                        NULL, NULL, TOKUDB_BACKUP_PLUGIN_VERSION_STRING);
+static MYSQL_SYSVAR_STR(plugin_version, tokudb_backup_plugin_version,
+    PLUGIN_VAR_NOCMDARG | PLUGIN_VAR_READONLY,
+    "version of the tokudb backup plugin",
+    NULL, NULL, TOKUDB_BACKUP_PLUGIN_VERSION_STRING);
 
 static char *tokudb_backup_version = (char *) tokubackup_version_string;
 
-static MYSQL_SYSVAR_STR(version, tokudb_backup_version, PLUGIN_VAR_NOCMDARG | PLUGIN_VAR_READONLY, "version of the tokutek backup library",
-                        NULL, NULL, NULL);
+static MYSQL_SYSVAR_STR(version, tokudb_backup_version,
+    PLUGIN_VAR_NOCMDARG | PLUGIN_VAR_READONLY,
+    "version of the tokutek backup library",
+    NULL, NULL, NULL);
 
-static MYSQL_THDVAR_ULONG(last_error, PLUGIN_VAR_THDLOCAL, "error from the last backup. 0 is success",
-                          NULL, NULL, 0 /*default*/, 0 /*min*/, ~0ULL /*max*/, 1 /*blocksize*/);
+static MYSQL_THDVAR_ULONG(last_error,
+    PLUGIN_VAR_THDLOCAL,
+    "error from the last backup. 0 is success",
+    NULL, NULL, 0, 0, ~0ULL, 1);
 
-static MYSQL_THDVAR_STR(last_error_string, PLUGIN_VAR_THDLOCAL + PLUGIN_VAR_MEMALLOC, "error string from the last backup", NULL, NULL, NULL);
+static void tokudb_backup_update_last_error_str(THD* thd,
+                                                struct st_mysql_sys_var* var,
+                                                void* var_ptr, const void* save);
 
-static MYSQL_THDVAR_STR(exclude, PLUGIN_VAR_THDLOCAL + PLUGIN_VAR_MEMALLOC, "exclude source file regular expression", NULL, NULL, NULL);
+static MYSQL_THDVAR_STR(last_error_string,
+    PLUGIN_VAR_THDLOCAL,
+    "error string from the last backup",
+    NULL, tokudb_backup_update_last_error_str, NULL);
 
-static int tokudb_backup_check_dir(THD *thd, struct st_mysql_sys_var *var, void *save, struct st_mysql_value *value);
-static void tokudb_backup_update_dir(THD *thd, struct st_mysql_sys_var *var, void *var_ptr, const void *save);
+static MYSQL_THDVAR_STR(exclude,
+    PLUGIN_VAR_THDLOCAL + PLUGIN_VAR_MEMALLOC,
+    "exclude source file regular expression",
+    NULL, NULL, NULL);
 
-static MYSQL_THDVAR_STR(dir, PLUGIN_VAR_THDLOCAL + PLUGIN_VAR_MEMALLOC, "name of the directory where the backup is stored", tokudb_backup_check_dir, tokudb_backup_update_dir, NULL);
+static int tokudb_backup_check_dir(THD* thd, struct st_mysql_sys_var* var,
+                                   void* save, struct st_mysql_value* value);
 
-static int tokudb_backup_check_throttle(THD *thd, struct st_mysql_sys_var *var, void *save, struct st_mysql_value *value);
-static void tokudb_backup_update_throttle(THD *thd, struct st_mysql_sys_var *var, void *var_ptr, const void *save);
+static void tokudb_backup_update_dir(THD* thd, struct st_mysql_sys_var* var,
+                                     void* var_ptr, const void* save);
 
-static MYSQL_THDVAR_ULONGLONG(throttle, PLUGIN_VAR_THDLOCAL, "backup throttle on write rate in bytes per second",
-                              tokudb_backup_check_throttle, tokudb_backup_update_throttle, ~0ULL /*default*/, 0 /*min*/, ~0ULL /*max*/, 1 /*blocksize*/);
+static MYSQL_THDVAR_STR(dir,
+    PLUGIN_VAR_THDLOCAL + PLUGIN_VAR_MEMALLOC,
+    "name of the directory where the backup is stored",
+    tokudb_backup_check_dir, tokudb_backup_update_dir, NULL);
+
+static int tokudb_backup_check_throttle(THD* thd, struct st_mysql_sys_var* var,
+                                        void* save, struct st_mysql_value* value);
+
+static void tokudb_backup_update_throttle(THD* thd, struct st_mysql_sys_var* var,
+                                          void* var_ptr, const void* save);
+
+static MYSQL_THDVAR_ULONGLONG(throttle,
+    PLUGIN_VAR_THDLOCAL,
+    "backup throttle on write rate in bytes per second",
+    tokudb_backup_check_throttle, tokudb_backup_update_throttle,
+    ~0ULL, 0, ~0ULL, 1);
 
 static char *tokudb_backup_allowed_prefix;
 
-static MYSQL_SYSVAR_STR(allowed_prefix, tokudb_backup_allowed_prefix, PLUGIN_VAR_READONLY, "allowed prefix of the destination directory",
-                        NULL, NULL, NULL);
+static MYSQL_SYSVAR_STR(allowed_prefix, tokudb_backup_allowed_prefix,
+    PLUGIN_VAR_READONLY,
+    "allowed prefix of the destination directory",
+    NULL, NULL, NULL);
 
 static struct st_mysql_sys_var *tokudb_backup_system_variables[] = {
     MYSQL_SYSVAR(plugin_version),
@@ -119,8 +149,12 @@ static int tokudb_backup_progress_fun(float progress, const char *progress_strin
 static void tokudb_backup_set_error(THD *thd, int error, const char *error_string) {
     THDVAR(thd, last_error) = error;
     char *old_error_string = THDVAR(thd, last_error_string);
-    THDVAR(thd, last_error_string) = error_string ? my_strdup(error_string, MYF(MY_FAE)) : NULL;
-    my_free(old_error_string);
+    if (error_string)
+        THDVAR(thd, last_error_string) = my_strdup(error_string, MYF(MY_FAE));
+    else
+        THDVAR(thd, last_error_string) = NULL;
+    if (old_error_string)
+        my_free(old_error_string);
 }
 
 static void tokudb_backup_set_error_string(THD *thd, int error, const char *error_fmt, const char *s1, const char *s2, const char *s3) {
@@ -132,6 +166,13 @@ static void tokudb_backup_set_error_string(THD *thd, int error, const char *erro
     my_free(error_string);
 }
 
+static void tokudb_backup_update_last_error_str(THD* thd,
+                                                struct st_mysql_sys_var* var,
+                                                void* var_ptr, const void* save) {
+    tokudb_backup_set_error(thd, THDVAR(thd, last_error), ((LEX_STRING*)save)->str);
+    *((char**)var_ptr) = THDVAR(thd, last_error_string);
+}
+
 struct tokudb_backup_error_extra {
     THD *_thd;
 };
@@ -521,7 +562,7 @@ static void tokudb_backup_run(THD *thd, const char *dest_dir) {
     }
     free(dest_dir_path);
 
-    struct source_dirs sources;
+    source_dirs sources;
     sources.find_and_allocate_dirs(thd);
 
     if (sources.check_dirs_layout(thd) == false) {
