@@ -1,7 +1,8 @@
 #ifndef SQL_STRING_INCLUDED
 #define SQL_STRING_INCLUDED
 
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights
+ * reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,51 +19,130 @@
 
 /* This file is originally from the mysql distribution. Coded by monty */
 
-#ifdef USE_PRAGMA_INTERFACE
-#pragma interface			/* gcc class implementation */
-#endif
-
 #include "m_ctype.h"                            /* my_charset_bin */
 #include "my_sys.h"              /* alloc_root, my_free, my_realloc */
 #include "m_string.h"                           /* TRASH */
 
-#define QUOTED_IDENTIFIER(str_name, buf_size, q, cs, id_name, id_size) \
-	char buf_##str_name[(buf_size)]; \
-	String str_name((buf_##str_name), (buf_size), (cs)); \
-	str_name.length(0); \
-	str_name.append_identifier((id_name), (id_size), (cs), (q));
+
+/**
+  A wrapper class for null-terminated constant strings.
+  Constructors make sure that the position of the '\0' terminating byte
+  in m_str is always in sync with m_length.
+
+  This class must stay as small as possible as we often 
+  pass it and its descendants (such as Name_string) into functions
+  using call-by-value evaluation.
+
+  Don't add new members or virual methods into this class!
+*/
+class Simple_cstring
+{
+private:
+  const char *m_str;
+  size_t m_length;
+protected:
+  /**
+    Initialize from a C string whose length is already known.
+  */
+  void set(const char *str_arg, size_t length_arg)
+  {
+    // NULL is allowed only with length==0
+    DBUG_ASSERT(str_arg || length_arg == 0);
+    // For non-NULL, make sure length_arg is in sync with '\0' terminator.
+    DBUG_ASSERT(!str_arg || str_arg[length_arg] == '\0');
+    m_str= str_arg;
+    m_length= length_arg;
+  }
+public:
+  Simple_cstring()
+  {
+    set(NULL, 0);
+  }
+  Simple_cstring(const char *str_arg, size_t length_arg)
+  {
+    set(str_arg, length_arg);
+  }
+  Simple_cstring(const LEX_STRING arg)
+  {
+    set(arg.str, arg.length);
+  }
+  void reset()
+  {
+    set(NULL, 0);
+  }
+  /**
+    Set to a null-terminated string.
+  */
+  void set(const char *str)
+  {
+    set(str, str ? strlen(str) : 0);
+  }
+  /**
+    Return string buffer.
+  */
+  const char *ptr() const { return m_str; }
+  /**
+    Check if m_ptr is set.
+  */
+  bool is_set() const { return m_str != NULL; }
+  /**
+    Return name length.
+  */
+  size_t length() const { return m_length; }
+  /**
+    Compare to another Simple_cstring.
+  */
+  bool eq_bin(const Simple_cstring other) const
+  {
+    return m_length == other.m_length &&
+           memcmp(m_str, other.m_str, m_length) == 0;
+  }
+  /**
+    Copy to the given buffer
+  */
+  void strcpy(char *buff) const
+  {
+    memcpy(buff, m_str, m_length);
+    buff[m_length]= '\0';
+  }
+};
+
 
 class String;
 typedef struct charset_info_st CHARSET_INFO;
 typedef struct st_io_cache IO_CACHE;
 typedef struct st_mem_root MEM_ROOT;
 
-int sortcmp(const String *a,const String *b, CHARSET_INFO *cs);
+int sortcmp(const String *a,const String *b, const CHARSET_INFO *cs);
 String *copy_if_not_alloced(String *a,String *b,uint32 arg_length);
-uint32 copy_and_convert(char *to, uint32 to_length, CHARSET_INFO *to_cs,
-			const char *from, uint32 from_length,
-			CHARSET_INFO *from_cs, uint *errors);
-uint32 well_formed_copy_nchars(CHARSET_INFO *to_cs,
+inline uint32 copy_and_convert(char *to, uint32 to_length,
+                               const CHARSET_INFO *to_cs,
+                               const char *from, uint32 from_length,
+                               const CHARSET_INFO *from_cs, uint *errors)
+{
+  return my_convert(to, to_length, to_cs, from, from_length, from_cs, errors);
+}
+uint32 well_formed_copy_nchars(const CHARSET_INFO *to_cs,
                                char *to, uint to_length,
-                               CHARSET_INFO *from_cs,
+                               const CHARSET_INFO *from_cs,
                                const char *from, uint from_length,
                                uint nchars,
                                const char **well_formed_error_pos,
                                const char **cannot_convert_error_pos,
                                const char **from_end_pos);
-size_t my_copy_with_hex_escaping(CHARSET_INFO *cs,
+size_t my_copy_with_hex_escaping(const CHARSET_INFO *cs,
                                  char *dst, size_t dstlen,
                                  const char *src, size_t srclen);
 uint convert_to_printable(char *to, size_t to_len,
                           const char *from, size_t from_len,
-                          CHARSET_INFO *from_cs, size_t nbytes= 0);
+                          const CHARSET_INFO *from_cs, size_t nbytes= 0);
 
 class String
 {
   char *Ptr;
   uint32 str_length,Alloced_length;
   bool alloced;
-  CHARSET_INFO *str_charset;
+  const CHARSET_INFO *str_charset;
 public:
   String()
   { 
@@ -74,17 +154,17 @@ public:
     alloced=0; Alloced_length=0; (void) real_alloc(length_arg); 
     str_charset= &my_charset_bin;
   }
-  String(const char *str, CHARSET_INFO *cs)
+  String(const char *str, const CHARSET_INFO *cs)
   { 
     Ptr=(char*) str; str_length=(uint) strlen(str); Alloced_length=0; alloced=0;
     str_charset=cs;
   }
-  String(const char *str,uint32 len, CHARSET_INFO *cs)
+  String(const char *str,uint32 len, const CHARSET_INFO *cs)
   { 
     Ptr=(char*) str; str_length=len; Alloced_length=0; alloced=0;
     str_charset=cs;
   }
-  String(char *str,uint32 len, CHARSET_INFO *cs)
+  String(char *str,uint32 len, const CHARSET_INFO *cs)
   { 
     Ptr=(char*) str; Alloced_length=str_length=len; alloced=0;
     str_charset=cs;
@@ -107,9 +187,9 @@ public:
   { /* never called */ }
   ~String() { free(); }
 
-  inline void set_charset(CHARSET_INFO *charset_arg)
+  inline void set_charset(const CHARSET_INFO *charset_arg)
   { str_charset= charset_arg; }
-  inline CHARSET_INFO *charset() const { return str_charset; }
+  inline const CHARSET_INFO *charset() const { return str_charset; }
   inline uint32 length() const { return str_length;}
   inline uint32 alloced_length() const { return Alloced_length;}
   inline char& operator [] (uint32 i) const { return Ptr[i]; }
@@ -167,20 +247,20 @@ public:
      @param cs Character set to use for interpreting string data.
      @note The new buffer will not be null terminated.
   */
-  inline void set(char *str,uint32 arg_length, CHARSET_INFO *cs)
+  inline void set(char *str,uint32 arg_length, const CHARSET_INFO *cs)
   {
     free();
     Ptr=(char*) str; str_length=Alloced_length=arg_length ; alloced=0;
     str_charset=cs;
   }
-  inline void set(const char *str,uint32 arg_length, CHARSET_INFO *cs)
+  inline void set(const char *str,uint32 arg_length, const CHARSET_INFO *cs)
   {
     free();
     Ptr=(char*) str; str_length=arg_length; Alloced_length=0 ; alloced=0;
     str_charset=cs;
   }
   bool set_ascii(const char *str, uint32 arg_length);
-  inline void set_quick(char *str,uint32 arg_length, CHARSET_INFO *cs)
+  inline void set_quick(char *str,uint32 arg_length, const CHARSET_INFO *cs)
   {
     if (!alloced)
     {
@@ -188,12 +268,12 @@ public:
     }
     str_charset=cs;
   }
-  bool set_int(longlong num, bool unsigned_flag, CHARSET_INFO *cs);
-  bool set(longlong num, CHARSET_INFO *cs)
+  bool set_int(longlong num, bool unsigned_flag, const CHARSET_INFO *cs);
+  bool set(longlong num, const CHARSET_INFO *cs)
   { return set_int(num, false, cs); }
-  bool set(ulonglong num, CHARSET_INFO *cs)
+  bool set(ulonglong num, const CHARSET_INFO *cs)
   { return set_int((longlong)num, true, cs); }
-  bool set_real(double num,uint decimals, CHARSET_INFO *cs);
+  bool set_real(double num,uint decimals, const CHARSET_INFO *cs);
 
   /*
     PMG 2004.11.12
@@ -219,7 +299,9 @@ public:
   */
   inline void chop()
   {
-    Ptr[str_length--]= '\0'; 
+    str_length--;
+    Ptr[str_length]= '\0';
+    DBUG_ASSERT(strlen(Ptr) == str_length);
   }
 
   inline void free()
@@ -240,7 +322,7 @@ public:
     return real_alloc(arg_length);
   }
   bool real_alloc(uint32 arg_length);			// Empties old string
-  bool realloc(uint32 arg_length);
+  bool realloc(uint32 arg_length, bool force_on_heap= false);
 
   // Shrink the buffer, but only if it is allocated on the heap.
   inline void shrink(uint32 arg_length)
@@ -279,37 +361,64 @@ public:
     }
     return *this;
   }
+  /**
+    Takeover the buffer owned by another string.
+    "this" becames the owner of the buffer and
+    is further responsible to free it.
+    The string "s" is detouched from the buffer (cleared).
+
+    @param s - a String object to steal buffer from.
+  */
+  inline void takeover(String &s)
+  {
+    DBUG_ASSERT(this != &s);
+    // Make sure buffers of the two Strings do not overlap
+    DBUG_ASSERT(!s.uses_buffer_owned_by(this));
+    free();
+    Ptr= s.Ptr;
+    str_length= s.str_length;
+    Alloced_length= s.Alloced_length;
+    alloced= s.alloced;
+    str_charset= s.str_charset;
+    s.Ptr= NULL;
+    s.Alloced_length= 0;
+    s.str_length= 0;
+    s.alloced= 0;
+  }
 
   bool copy();					// Alloc string if not alloced
   bool copy(const String &s);			// Allocate new string
-  bool copy(const char *s,uint32 arg_length, CHARSET_INFO *cs);	// Allocate new string
+  // Allocate new string
+  bool copy(const char *s,uint32 arg_length, const CHARSET_INFO *cs);
   static bool needs_conversion(uint32 arg_length,
-  			       CHARSET_INFO *cs_from, CHARSET_INFO *cs_to,
+  			       const CHARSET_INFO *cs_from, const CHARSET_INFO *cs_to,
 			       uint32 *offset);
   static bool needs_conversion_on_storage(uint32 arg_length,
-                                          CHARSET_INFO *cs_from,
-                                          CHARSET_INFO *cs_to);
+                                          const CHARSET_INFO *cs_from,
+                                          const CHARSET_INFO *cs_to);
   bool copy_aligned(const char *s, uint32 arg_length, uint32 offset,
-		    CHARSET_INFO *cs);
-  bool set_or_copy_aligned(const char *s, uint32 arg_length, CHARSET_INFO *cs);
-  bool copy(const char*s,uint32 arg_length, CHARSET_INFO *csfrom,
-	    CHARSET_INFO *csto, uint *errors);
+		    const CHARSET_INFO *cs);
+  bool set_or_copy_aligned(const char *s, uint32 arg_length,
+                           const CHARSET_INFO *cs);
+  bool copy(const char*s,uint32 arg_length, const CHARSET_INFO *csfrom,
+	    const CHARSET_INFO *csto, uint *errors);
   bool append(const String &s);
   bool append(const char *s);
   bool append(LEX_STRING *ls)
   {
-    return append(ls->str, ls->length);
+    return append(ls->str, (uint32) ls->length);
+  }
+  bool append(Simple_cstring str)
+  {
+    return append(str.ptr(), static_cast<uint>(str.length()));
   }
   bool append(const char *s, uint32 arg_length);
-  bool append(const char *s, uint32 arg_length, CHARSET_INFO *cs);
+  bool append(const char *s, uint32 arg_length, const CHARSET_INFO *cs);
   bool append_ulonglong(ulonglong val);
   bool append(IO_CACHE* file, uint32 arg_length);
   bool append_with_prefill(const char *s, uint32 arg_length, 
 			   uint32 full_length, char fill_char);
-  bool append_identifier(const char *name,
-			 uint length,
-			 CHARSET_INFO *ci,
-			 int quote_char);
+  bool append_parenthesized(long nr, int radix= 10);
   int strstr(const String &search,uint32 offset=0); // Returns offset to substring or -1
   int strrstr(const String &search,uint32 offset=0); // Returns offset to substring or -1
   bool replace(uint32 offset,uint32 arg_length,const char *to,uint32 length);
@@ -330,10 +439,10 @@ public:
   }
   bool fill(uint32 max_length,char fill);
   void strip_sp();
-  friend int sortcmp(const String *a,const String *b, CHARSET_INFO *cs);
+  friend int sortcmp(const String *a,const String *b, const CHARSET_INFO *cs);
   friend int stringcmp(const String *a,const String *b);
   friend String *copy_if_not_alloced(String *a,String *b,uint32 arg_length);
-  uint32 numchars();
+  uint32 numchars() const;
   int charpos(int i,uint32 offset=0);
 
   int reserve(uint32 space_needed)
@@ -341,7 +450,6 @@ public:
     return realloc(str_length + space_needed);
   }
   int reserve(uint32 space_needed, uint32 grow_by);
-
   /*
     The following append operations do NOT check alloced memory
     q_*** methods writes values of parameters itself
@@ -413,7 +521,6 @@ public:
     return FALSE;
   }
   void print(String *print);
-  void append_for_single_quote(const char *st, uint len);
 
   /* Swap two string objects. Efficient way to exchange data without memcpy. */
   void swap(String &s);
@@ -435,15 +542,66 @@ public:
     }
     return TRUE;
   }
+  /**
+    Make a zero-terminated copy of our value,allocated in the specified MEM_ROOT
+
+    @param root         MEM_ROOT to allocate the result
+
+    @return allocated string or NULL
+  */
+  char *dup(MEM_ROOT *root) const
+  {
+    if (str_length > 0 && Ptr[str_length - 1] == 0)
+      return static_cast<char *>(memdup_root(root, Ptr, str_length));
+
+    char *ret= static_cast<char*>(alloc_root(root, str_length + 1));
+    if (ret != NULL)
+    {
+      memcpy(ret, Ptr, str_length);
+      ret[str_length]= 0;
+    }
+    return ret;
+  }
 };
 
-static inline bool check_if_only_end_space(CHARSET_INFO *cs, char *str, 
+
+/**
+  String class wrapper with a preallocated buffer of size buff_sz
+
+  This class allows to replace sequences of:
+     char buff[12345];
+     String str(buff, sizeof(buff));
+     str.length(0);
+  with a simple equivalent declaration:
+     StringBuffer<12345> str;
+*/
+
+template<size_t buff_sz>
+class StringBuffer : public String
+{
+  char buff[buff_sz];
+
+public:
+  StringBuffer() : String(buff, buff_sz, &my_charset_bin) { length(0); }
+  explicit StringBuffer(const CHARSET_INFO *cs) : String(buff, buff_sz, cs)
+  {
+    length(0);
+  }
+  StringBuffer(const char *str, size_t length, const CHARSET_INFO *cs)
+    : String(buff, buff_sz, cs)
+  {
+    set(str, length, cs);
+  }
+};
+
+
+static inline bool check_if_only_end_space(const CHARSET_INFO *cs, char *str, 
                                            char *end)
 {
   return str+ cs->cset->scan(cs, str, end, MY_SEQ_SPACES) == end;
 }
 
 bool
-validate_string(CHARSET_INFO *cs, const char *str, uint32 length,
+validate_string(const CHARSET_INFO *cs, const char *str, uint32 length,
                 size_t *valid_length, bool *length_error);
 #endif /* SQL_STRING_INCLUDED */
